/*
 * spree
 *
 * sends arena messages with optional bong depending on how many
 *  consecutive kills a player has. simple ?spree command.
 *
 * [Spree]
 * ResetSpreeOnFrequencyChange=0
 * ResetSpreeOnShipChange=0
 * SpreeEndMinimum=4
 * SpreeEndMessage=has had their spree cut short
 * SpreeEndBong=22
 * Message3=is dominating (3:0)
 * Bong3=13
 * PlayerObject3=100
 * ArenaObject3=101
 *
 * 15/02/04 - Created by Smong
 *
 * 16/02/04 - Added stdio.h
 *
 * 17/02/04 - added ?spree, get a player's current and best spree or
 *  display all people in the current arena on a spree.
 *
 * 26/02/04 - added %macros: killer, killed, bounty, coord
 *  added ResetSpreeOnFlagWin, default is off
 *
 * 14/03/04 - attempted fix for the %macro support.
 *
 * 21/03/04 - added last spree stat.
 *
 * 03/08/04 - updated prototype for Kill().
 *
 * 24/11/04 - updated to asss 1.3.2.
 *  made command arena specific.
 *  added PlayerObjectN and ArenaObjectN support (i think i heard someone say
 *   they wanted this).
 *
 */

#include <stdio.h>  //sprintf
#include <stdlib.h> //strtol
#include <string.h> //strstr, strncat, strcat, strlen, strcpy

#include "asss.h"
#include "objects.h"

typedef struct
{
	u16 wins;
	u16 best;
	u16 last;
} pdata;

local void replace_str(char *str, const char *key, const char *replacewith);

local void PlayerAction(Player *p, int action, Arena *arena);
local void Kill(Arena *arena, Player *killer, Player *killed,
	int bounty, int flagscarried, int *pts, int *green);
local void FreqChange(Player *p, int newfreq);
local void ShipChange(Player *p, int newship, int newfreq);
local void FlagWin(Arena *arena, int freq);

local void Cspree(const char *command, const char *params,
	Player *p, const Target *target);

local int pdkey;

local Imodman     *mm;
local Iplayerdata *pd;
local Iconfig     *cfg;
local Ichat       *chat;
local Icmdman     *cmd;
local Iobjects    *objs;

local void replace_str(char *str, const char *key, const char *replacewith)
{
	char buf[256];
	const char *found, *start;
	if (key == NULL || replacewith == NULL)
		return;
	
	while(1)
	{
		buf[0] = '\0';
		start = str;
	
		found = strstr(start, key);
		if (found == NULL) break;
		
		/* preserve preceding text */
		strncat(buf, start, found - start);
		
		/* append new the replacement text */
		strcat(buf, replacewith);
		
		/* append leftover text */
		start = found + strlen(key);
		strncat(buf, start, strlen(start));

		strcpy(str, buf);
	};
}

local void PlayerAction(Player *p, int action, Arena *arena)
{
	pdata *d;

	if (action == PA_ENTERARENA)
	{
		d = PPDATA(p, pdkey);
		d->wins = 0;
		d->best = 0;
		d->last = 0;
	}
}

local void Kill(Arena *arena, Player *killer, Player *killed,
	int bounty, int flagscarried, int *pts, int *green)
{
	char sound, key[16], buf[256];
	const char *str;
	pdata *d;

	d = PPDATA(killed, pdkey);
	if (d->wins >= cfg->GetInt(arena->cfg, "Spree", "SpreeEndMinimum", 65535))
	{
		/* spree end */
		str = cfg->GetStr(arena->cfg, "Spree", "SpreeEndMessage");
		if (str)
		{
			sound = (char)cfg->GetInt(arena->cfg, "Spree", "SpreeEndBong", 0);
			chat->SendArenaSoundMessage(arena, sound, "%s %s", killed->name, str);
		}
	}
	
	/* store last spree */
	d->last = d->wins;
	
	/* reset killed stat */
	d->wins = 0;
	
	/* increment killer stat */
	d = PPDATA(killer, pdkey);
	d->wins++;
	
	/* update record stat */
	if (d->wins > d->best)
		d->best = d->wins;

	/* get message */
	sprintf(key, "Message%d", d->wins);
	str = cfg->GetStr(arena->cfg, "Spree", key);
	if (!str) return;
	
	strncpy(buf, str, sizeof(buf));
	replace_str(buf, "%killer", killer->name);
	replace_str(buf, "%killed", killed->name);
	
	sprintf(key, "%d", killer->position.bounty);
	replace_str(buf, "%bounty", key);
	
	sprintf(key, "%c%d", killer->position.x/820 + 'A', killer->position.y/820);
	replace_str(buf, "%coord", key);
	
	/* get bong */
	sprintf(key, "Bong%d", d->wins);
	sound = (char)cfg->GetInt(arena->cfg, "Spree", key, 0);
	
	/* send message */
	chat->SendArenaSoundMessage(arena, sound, "%s", buf);

	if (objs)
	{
		/* toggle lvz */
		Target tgt;
		int id;

		/* personal object */
		sprintf(key, "PlayerObject%d", d->wins);
		id = cfg->GetInt(arena->cfg, "Spree", key, -1);

		if (id != -1)
		{
			tgt.u.p = killer;
			tgt.type = T_PLAYER;
			objs->Toggle(&tgt, id, TRUE);
		}

		/* arena object */
		sprintf(key, "ArenaObject%d", d->wins);
		id = cfg->GetInt(arena->cfg, "Spree", key, -1);

		if (id != -1)
		{
			tgt.u.arena = arena;
			tgt.type = T_ARENA;
			objs->Toggle(&tgt, id, TRUE);
		}
	}
}

local void FreqChange(Player *p, int newfreq)
{
	if (cfg->GetInt(p->arena->cfg, "Spree", "ResetSpreeOnFrequencyChange", 0))
	{
		pdata *d = PPDATA(p, pdkey);
		d->last = d->wins;
		d->wins = 0;
	}
}

local void ShipChange(Player *p, int newship, int newfreq)
{
	if (cfg->GetInt(p->arena->cfg, "Spree", "ResetSpreeOnShipChange", 0))
	{
		pdata *d = PPDATA(p, pdkey);
		d->last = d->wins;
		d->wins = 0;
	}
}

local void FlagWin(Arena *arena, int freq)
{
	if (cfg->GetInt(arena->cfg, "Spree", "ResetSpreeOnFlagWin", 0))
	{
		Link *link;
		Player *p;
		pdata *d;
	
		pd->Lock();
		FOR_EACH_PLAYER_P(p, d, pdkey)
		{
			if (p->arena != arena) continue;
			
			d->last = d->wins;
			d->wins = 0;
		}
		pd->Unlock();
	}
}

local helptext_t spree_help =
	"Module: spree\n"
	"Targets: arena, player\n"
	"Args: [<#>]\n"
	"Prints out the target player's current spree and best spree,\n"
	"if no target, yourself, or all players in the current arena\n"
	"with the specified spree or higher\n";

local void Cspree(const char *command, const char *params,
	Player *p, const Target *target)
{
	pdata *d;

	if (target->type == T_ARENA)
	{
		int min = strtol(params, NULL, 0);
		
		if (min > 0)
		{
			Player *t;
			Link *link;
			
			pd->Lock();
			FOR_EACH_PLAYER_P(t, d, pdkey)
			{
				if (t->arena != p->arena) continue;
				if (d->wins < min) continue;
				
				chat->SendMessage(p, "%s spree: current %d best"
					" %d last %d", t->name, d->wins,
					d->best, d->last);
			}
			pd->Unlock();
		}
		else
		{
			//Show player stats
			d = PPDATA(p, pdkey);
			chat->SendMessage(p, "spree: current %d best %d last %d",
				d->wins, d->best, d->last);
		}
	}
	else if (target->type == T_PLAYER)
	{
		/* show target stats */
		d = PPDATA(target->u.p, pdkey);
		chat->SendMessage(p, "%s spree: current %d best %d last %d",
			target->u.p->name, d->wins, d->best, d->last);
	}
}

EXPORT const char info_spree[] = "v1.5 smong <soinsg@hotmail.com>";

EXPORT int MM_spree(int action, Imodman *mm_, Arena *arena)
{
	if (action == MM_LOAD)
	{
		mm = mm_;
		pd       = mm->GetInterface(I_PLAYERDATA, ALLARENAS);
		cfg      = mm->GetInterface(I_CONFIG,     ALLARENAS);
		chat     = mm->GetInterface(I_CHAT,       ALLARENAS);
		cmd      = mm->GetInterface(I_CMDMAN,     ALLARENAS);
		objs     = mm->GetInterface(I_OBJECTS,    ALLARENAS);
		if (!pd || !cfg || !chat || !cmd)
			return MM_FAIL;

		pdkey = pd->AllocatePlayerData(sizeof(pdata));
		if (pdkey == -1) return MM_FAIL;

		return MM_OK;
	}
	else if (action == MM_UNLOAD)
	{
		pd->FreePlayerData(pdkey);

		mm->ReleaseInterface(pd);
		mm->ReleaseInterface(cfg);
		mm->ReleaseInterface(chat);
		mm->ReleaseInterface(cmd);
		mm->ReleaseInterface(objs);
		return MM_OK;
	}
	else if (action == MM_ATTACH)
	{
		mm->RegCallback(CB_PLAYERACTION, PlayerAction, arena);
		mm->RegCallback(CB_KILL,         Kill,         arena);
		mm->RegCallback(CB_FREQCHANGE,   FreqChange,   arena);
		mm->RegCallback(CB_SHIPCHANGE,   ShipChange,   arena);
		mm->RegCallback(CB_FLAGWIN,      FlagWin,      arena);

		cmd->AddCommand("spree", Cspree, arena, spree_help);

		return MM_OK;
	}
	else if (action == MM_DETACH)
	{
		cmd->RemoveCommand("spree", Cspree, arena);

		mm->UnregCallback(CB_PLAYERACTION, PlayerAction, arena);
		mm->UnregCallback(CB_KILL,         Kill,         arena);
		mm->UnregCallback(CB_FREQCHANGE,   FreqChange,   arena);
		mm->UnregCallback(CB_SHIPCHANGE,   ShipChange,   arena);
		mm->UnregCallback(CB_FLAGWIN,      FlagWin,      arena);
	
		return MM_OK;
	}

	return MM_FAIL;
}
